<?php
/*
 * Clip Name: WP Clips v3 File Upgrader
 * Description: This Clip adds required file and directory upgrades when updating to WP Clips version 3.
 * Version: 1.0
 * Author: WP Clips
 * Author URI: https://wpclips.net
 * Network: true
 *
 * @package WP_Clips/Precoded
 * @subpackage WP_Clips_v3_File_Upgrader
 */

class WP_Clips_v3_File_Upgrades {


	public function __construct() {

		if( ! is_admin() ) return;

		if( ! is_multisite() ) $this->wpc_v3_file_upgrades();
		else $this->wpc_multisite_v3_file_upgrades();

		$this->unclip();
	}

	// File upgrades for single site
	private function wpc_v3_file_upgrades() {

		// Rename custom-jquery.js file
		$files = [
				WP_CLIPS_ROOT . 'clip-custom/custom-jquery.js',
				WP_CLIPS_ROOT . 'precoded/unclipped/clip-custom/custom-jquery.js'
			];

		foreach( $files as $file )
			if( file_exists( $file ) ) {
				rename( $file, str_replace( 'custom-jquery.js', 'custom-script.js', $file ) );
			}

		// Upgrade custom and clip folders
		foreach( [ 'core', 'custom' ] as $clip ) {

			// Copy across core/custom clips
			if( file_exists( WP_CLIPS_ROOT . 'clip_' . $clip ) ) {
				if( file_exists( WP_CLIPS_DIR . 'clip-' . $clip ) ) {
					$this->delete( WP_CLIPS_DIR . 'clip-' . $clip );
				}
				rename( WP_CLIPS_ROOT . 'clip_' . $clip, WP_CLIPS_DIR . 'clip-' . $clip );
			}
			if( file_exists( WP_CLIPS_ROOT . 'precoded/unclipped/clip_' . $clip ) ) {
				if( file_exists( WP_CLIPS_UNCL . 'clip-' . $clip ) ) {
					$this->delete( WP_CLIPS_UNCL . 'clip-' . $clip );
				}
				rename( WP_CLIPS_ROOT . 'precoded/unclipped/clip_' . $clip, WP_CLIPS_UNCL . 'clip-' . $clip );
			}
		}

		// Upgrade unclipped folders
		if( file_exists( WP_CLIPS_ROOT . 'precoded/unclipped' ) ) {
			foreach( glob( WP_CLIPS_ROOT . 'precoded/unclipped/*' ) as $file ) {
				$dest = WP_CLIPS_UNCL . basename( $file );
				if( file_exists( $dest ) )
					$this->delete( $dest );
				rename( $file, $dest );
			}
			$this->delete( WP_CLIPS_ROOT . 'precoded/unclipped' );
		}

		// Upgrade precoded folders
		if( file_exists( WP_CLIPS_ROOT . 'precoded' ) ) {
			foreach( glob( WP_CLIPS_ROOT . 'precoded/*' ) as $file ) {
				$dest = WP_CLIPS_PREC . basename( $file );
				if( file_exists( $dest ) )
					$this->delete( $dest );
				rename( $file, $dest );
			}
			$this->delete( WP_CLIPS_ROOT . 'precoded' );
		}

		add_action( 'admin_print_footer_scripts', [ $this, 'complete' ] );
	}

	// File directory upgrades for multisite
	private function wpc_multisite_v3_file_upgrades() {

		foreach( glob( WP_CLIPS_ROOT . 'precoded_*/' ) as $dir ) {

			$id = substr( basename( $dir ), 9 );
			$site = $id ? WP_CLIPS_SITE . $id : WP_CLIPS_NETW;
			$clip = $id ? '/clip-custom' : '/clip-core';

			// Rename custom-jquery.js file
			if( $id ) {
				$files = [
					WP_CLIPS_ROOT . 'clip_' . $id . '/custom-jquery.js',
					WP_CLIPS_ROOT . 'precoded_' . $id . '/unclipped/clip_' . $id . '/custom-jquery.js'
				];

				foreach( $files as $file )
					if( file_exists( $file ) )
						rename( $file, str_replace( 'custom-jquery.js', 'custom-script.js', $file ) );
			}

			// Copy across core/custom clips
			if( file_exists( WP_CLIPS_ROOT . 'clip_' . $id ) ) {
				if( file_exists( $site . $clip ) ) {
					$this->delete( $site . $clip );
				}
				rename( WP_CLIPS_ROOT . 'clip_' . $id, $site . $clip );
			}
			if( file_exists( $dir . 'unclipped/clip_' . $id ) ) {
				if( file_exists( $site . '/unclipped' . $clip ) ) {
					$this->delete( $site . '/unclipped' . $clip );
				}
				rename( $dir . 'unclipped/clip_' . $id, $site . '/unclipped' . $clip );
			}

			// Copy across unclipped folders
			if( file_exists( $dir . 'unclipped' ) ) {
				foreach( glob( $dir . 'unclipped/*' ) as $file ) {
					$dest = $site . '/unclipped/' . basename( $file );
					if( file_exists( $dest ) )
						$this->delete( $dest );
					rename( $file, $dest );
				}
				$this->delete( $dir . 'unclipped/' ); 
			}

			// Copy across precoded folders
			if( file_exists( $dir ) ) {
				foreach( glob( $dir . '*' ) as $file ) {
					$dest = $site . '/precoded/' . basename( $file );
					if( file_exists( $dest ) )
						$this->delete( $dest );
					rename( $file, $dest );
				}
				$this->delete( $dir );
			}
		}

		add_action( 'admin_print_footer_scripts', [ $this, 'complete' ] );
	}

	// Complete message
	public function complete() {

		printf( '<div class="notice notice-success"><p>%s. <strong>%s.</strong></p></div>',
					__( 'Clips have been successfully upgraded to version 3', 'wp-clip' ),
					__( 'The upgrade Clip can now be deleted', 'wp-clips' )
				);
		printf( '<div class="notice notice-info"><p><strong>%s</strong> %s <a href="https://wpclips.net/customizing/" target="_blank">%s</a> %s.</p></div>',
					__( 'Important!', 'wp-clips' ),
					__( 'Clip dependencies are no longer declared in vitals.php files. Refer to', 'wp-clips' ),
					__( 'Customizing a Clip', 'wp-clips' ),
					__( 'for the new protocol', 'wp-clips' )
				);
	}

	// Directory deletions
	private function delete( $dir ) {

		$files = array_diff( scandir( $dir ), [ '.', '..' ] );
    	foreach( $files as $file ) {
			$path = $dir . '/' . $file;
			is_dir( $path ) ? $this->delete( $path ) : unlink( $path );
		}
		return rmdir( $dir );
	}

	// Self unclip file
	private function unclip() {

		$self = dirname( __FILE__ );
		rename( $self, str_replace( '/precoded/', '/unclipped/', $self ) );
	}
}

new WP_Clips_v3_File_Upgrades();